package johndog.core;

import java.io.*;
import java.util.*;
import java.util.jar.*;
import java.text.*;

import johndog.util.*;

class JDHTTPResponse {
	private static final List IMPL_METHODS = Arrays.asList(new String[]{"GET", "POST"});
	private static final JDHTTPAction HTTP_ACTION = new JDHTTPAction();
	private int code;
	private String website;
	private String path;
	private String method;
	private String content;
	
	public JDHTTPResponse() {
		code = 200;
		website = null;
		path = null;
		method = null;
		content = null;
	}

	public void validateRequest(String method, String path, String protocol) {
		if (method == null || path == null || protocol == null)
			code = 400;
		else if (!protocol.equals("HTTP/1.1"))
			code = 505;
		else if (!path.startsWith("/"))
			code = 400;
		else if (!IMPL_METHODS.contains(method))
			code = 501;
			
		if (code == 200)
			this.method = method;
	}

	public void validatePath(String path, Map<String, String> headers) {
		if (code == 200) {
			String filesep = System.getProperty("file.separator");
			String host = headers.get("Host");
			
			if (host != null && host.indexOf(":") != -1)
				host = host.substring(0, host.indexOf(":")); //discard port if any
			if (host == null || !host.toLowerCase().matches("^([a-z0-9][-\\.]?)*[a-z0-9]$")) {
				code = 400;
				return;
			}
			
			String website = JDCoreConfig.WEBROOT + filesep + host + ".jar";
			website.replace(filesep + filesep, filesep);
			if (path.endsWith("/")) {
				path += JDCoreConfig.DEFINDEX;
			}
			path.replace("/", filesep);
			path = path.substring(1, path.length());
			
			try {
				/* Validate host jar file */
				File file = new File(website);
				if (!file.exists()) {
					code = 404;
					return;
				}
				if (!file.getCanonicalPath().startsWith(JDCoreConfig.WEBROOT + filesep) || !file.isFile()) {
					code = 403;
					return;
				}
				
				/* Validate requested path */
				JarFile jar = new JarFile(file);
				JarEntry je = jar.getJarEntry(path + ".class");
				if (je == null) {
					code = 404;
					return;
				}				
				if (je.isDirectory()) {
					code = 403;
					return;					
				}
				
				this.website = website;
				this.path = path;
			} catch (IOException ex) {
				code = 503;
			}
		}
	}
	
	public void getResource(Map<String, String> headers, Map<String, String> params) {
		if (code == 200) {
			JDHTTPContext context = new JDHTTPContext();
			context.addHeaders(headers);
			context.addParams(params.get(method));
			try {
				String javapath = path;
				javapath.replace(System.getProperty("file.separator"), ".");
				
				JDClassLoader jdcl = (JDClassLoader)this.getClass().getClassLoader();
				jdcl.setSiteBase(website);
				
				context.loadWebSite(jdcl.loadClass(javapath), method);
				content = context.getValue().toString();
			} catch (Throwable ex) {
				code = 500;
				JDUtil.jdDebug("Internal Server Error: " + ex.toString());
			}
		}
	}
	
	public void doResponse(DataOutputStream out) {
		try {			
			String status = HTTP_ACTION.getStatus(code);
			out.writeBytes("HTTP/1.1 " + String.valueOf(code) + " " + status + "\r\n");
			out.writeBytes("Date: " + (new SimpleDateFormat(JDUtil.DATEFMT, Locale.US)).format(new Date()) + "\r\n");
			out.writeBytes("Server: " + JDUtil.JDVERSION + "\r\n");
			
			String content = HTTP_ACTION.getContent(code);
			if (content == null)
				content = this.content;
			if (content != null) {
				out.writeBytes("Content-Type: text/html\r\n");
				out.writeBytes("Content-Length: " + String.valueOf(content.length()) + "\r\n");
			}
			out.writeBytes("\r\n");
			if (content != null)
				out.writeBytes(content);
		} catch (IOException ex) {}
	}

}
